package com.auth.dao.base.entity.base;

import com.baomidou.mybatisplus.annotation.*;
import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.Accessors;
import org.springframework.security.core.CredentialsContainer;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.time.LocalDateTime;
import java.util.Date;
import java.util.Set;

@Getter
@Setter
@Accessors(chain = true)
@TableName("sys_user")
@Schema(name = "User对象", title = "用户信息", description = "用户信息的实体类对象")
public class UserEntity implements UserDetails, CredentialsContainer {

    @Schema(name = "id", title = "唯一标识")
    @TableId(value = "id", type = IdType.ASSIGN_ID)
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long id;

    @Schema(name = "username", title = "用户名")
    private String username;

    @Schema(name = "nickname", title = "昵称")
    private String nickname;

    @Schema(name = "email", title = "邮箱")
    private String email;

    @Schema(name = "phone", title = "手机号")
    private String phone;

    @Schema(name = "password", title = "密码")
    private String password;

    @Schema(name = "avatar", title = "头像")
    private String avatar;

    @Schema(name = "sex", title = "0:女 1:男")
    private Integer sex;

    @Schema(name = "summary", title = "个人描述")
    private String summary;

    @Schema(name = "ipAddress", title = "最后登录IP")
    private String ipAddress;

    @Schema(name = "ipRegion", title = "最后登录ip归属地")
    private String ipRegion;

    @Schema(name = "status", title = "1:禁用 0:正常")
    private Integer status;

    @Schema(name = "isDeleted", title = "是否删除")
    private Integer isDeleted;

    @Schema(name = "salt", title = "密码盐值")
    private String salt;

    @Schema(name = "gender", title = "性别(0=未知,1=男,2=女)")
    private Integer gender;

    @Schema(name = "birthday", title = "出生日期")
    private Date birthday;

    @Schema(name = "introduction", title = "个人简介")
    private String introduction;

    @Schema(name = "lastLoginIp", title = "最后登录IP")
    private String lastLoginIp;

    @Schema(name = "lastLoginRegion", title = "最后登录地区")
    private String lastLoginRegion;

    @Schema(name = "lastLoginTime", title = "最后登录时间")
    private LocalDateTime lastLoginTime;

    @Schema(name = "deptId", title = "所属部门ID")
    private Long deptId;

    @Schema(name = "postId", title = "所属岗位ID")
    private Long postId;

    @Schema(name = "createTime", title = "创建时间")
    @TableField(fill = FieldFill.INSERT)
    private LocalDateTime createTime;

    @Schema(name = "updateTime", title = "更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private LocalDateTime updateTime;

    @Schema(name = "createUser", title = "创建用户")
    @TableField(fill = FieldFill.INSERT)
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long createUser;

    @Schema(name = "updateUser", title = "操作用户")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @JsonSerialize(using = ToStringSerializer.class)
    private Long updateUser;

    @TableField(exist = false)
    private Set<? extends GrantedAuthority> authorities;

    @Override
    public void eraseCredentials() {
        this.password = null;
    }

    @Override
    public Set<? extends GrantedAuthority> getAuthorities() {
        return this.authorities;
    }
}